#  su2gmsh.rb
#  
#  Copyright 2013 Alessandro Barracco <bomastudio@gmail.com>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
# 
# 
Sketchup::require 'sketchup'

module SU2GMSH
	class Float
		def round_to(x)
			(self*10**x).round.to_f/10**x
		end 
	end
	
	class Exporter
		def initialize
			param = ["lc param"]
			defaults = [0.1]
			lc = UI.inputbox(param,defaults,"TECLA: Gmsh exporter")
			@model = Sketchup.active_model
		    # get selected/all entities of the active model
			if @model.selection.empty?
				@entities = @model.entities
			else
				@entities = @model.selection
			end
		
			if @entities.length == 0
				UI.messagebox("Nothing to do!")
				return nil
			end
			# define the filename
			@modelName = "Untitled" if (modelName = File.basename(@model.path, ".skp")) == ""
			
			cr = "\n"   # variabile per il ritorno a capo nelle stringhes
			
			str = "/*********************************************************************" << cr
			str << "*" << cr
			str <<"*  Gmsh *.geo file" << cr
			str <<"*  generated by TECLA - (c) ing. Alessandro Barracco" << cr
			str <<"*" << cr
			str <<"*  Variables, elementary entities (points, lines, surfaces), physical" << cr
			str <<"*  entities (points, lines, surfaces)" << cr
			str <<"*" << cr
			str <<"*********************************************************************/" << cr
			str << cr
			str << "// The simplest construction in Gmsh's scripting language is the" << cr
			str << "// `affectation'. The following command defines a new variable `lc':" << cr
		
			lc = lc[0].to_f
			
			str << "lc = #{lc};" << cr
			@countpoint = 1
			str << "// This variable can then be used in the definition of Gmsh's simplest" << cr
			str << "// `elementary entity', a `Point'. A Point is defined by a list of" << cr
			str << "// four numbers: three coordinates (X, Y and Z), and a characteristic" << cr
			str << "// length (lc) that sets the target element size at the point:" << cr
			# ------------------------------------------------------------------
			# NODES
			# ------------------------------------------------------------------
			@nodes = []
			@entities.each {|e|
				if e.class == Sketchup::Edge
					@vertices = e.vertices
					0.upto(@vertices.length - 1) do |v|
						x = @vertices[v].position.x.to_m
						y = @vertices[v].position.y.to_m
						z = @vertices[v].position.z.to_m
						@nodes << [x,y,z]	
					end
				end
			}
			@nodes.uniq! # rimuovo i vertici duplicati
			0.upto(@nodes.length - 1) do |n|
				x = @nodes[n][0]
				y = @nodes[n][1]
				z = @nodes[n][2]
				str << "Point(#{@countpoint}) = {#{x}, #{y}, #{z}, lc}"
				str << ";" << cr
				@countpoint += 1
			end
			# ------------------------------------------------------------------
	

		
			# ------------------------------------------------------------------
			# EDGES
			# ------------------------------------------------------------------
			@lati = []
			@edges = []
			@entities.each {|e|
				if e.class == Sketchup::Edge
					@lati << e
				end
			}
			@countline=1
			0.upto(@lati.length - 1) do |v|
				str << "Line(#{@countline}) = {"
				v1 = @lati[v].start.position # vertice iniziale
				v2 = @lati[v].end.position # vertice finale
				# conversione delle coordinate x,y,z di ogni nodo da m a float
				v1 = [v1.x.to_m, v1.y.to_m, v1.z.to_m]
				v2 = [v2.x.to_m, v2.y.to_m, v2.z.to_m]
				# ricerca dei nodi corrispondenti....
				0.upto(@nodes.length - 1) do |m|
					if @nodes[m] == v1
						@n1 = m + 1
						str << "#{@n1},"
					end
				end
				0.upto(@nodes.length - 1) do |m|
					if @nodes[m] == v2
						@n2 = m + 1
						str << "#{@n2}"
					end
				end
				str << "};" << cr
				
				@edges << [@countline, @lati[v], @n1, @n2]
				@countline += 1
			end
			puts @edges.inspect
			puts "*"*50
			# ------------------------------------------------------------------
		
			# ------------------------------------------------------------------
			# LOOP esterno
			# ------------------------------------------------------------------
			
=begin
	THE FOLLOWING IS THE PART I'M NOT ABLE TO SOLVE.... a GMSH user tried to help me and send me this post:
	
	"if you know which edges should form your line loop and you know the starting
	and ending vertex number for each of these edges, then it should be quite
	simple:
	  1) pick the first edge from a loop
	
	  2) the following edge must be such that its starting vertex number is the
	same as the ending vertex number of the previous edge. If that's not the case,
	add the new edge in the for loop with a minus sign (i.e. flip it)
	 Here I suppose that the new edge you're adding has at least one vertex common
	with the prevous edge
	
	 3) if there are any edges left to process (the loop is still not closed),
	repeat 2)"
	
	I CAN IDENTIFY ALL THE EDGES OF THE LOOP but with SIGN NOT CORRECT!
	
=end
			str << "// A line loop is a list of" << cr
			str << "// connected lines, a sign being associated with each line (depending" << cr
			str << "// on the orientation of the line):" << cr
			@loop=[]
			@entities.each {|e|	 # e = entità generica
				if e.class == Sketchup::Face
					str << "Line Loop(#{@countline}) = {"
					@loop << @countline
					bordiesterni = e.outer_loop.edges
					#~ @usocontorno = e.outer_loop.edgeuses
					
					#~ 
					#~ for i in 0..@usocontorno.length-1
						#~ if @usocontorno[i].reversed? #bordo col segno "-"
							#~ # str << "-#{@edges[i][1]},"
							#~ str << "-#{@edges[i][0]},"
						#~ else # bordo col segno "+"
							#~ # str << "#{@edges[i][1]}," 
							#~ str << "#{@edges[i][0]},"
						#~ end
					#~ end

					bordiesterni.each{|e|
						n=0
						@edges.each{|f|
							if f[n][1]==e
								n1=f[n][2]
								n2=f[n][3]
								break
							else
								next
							end
						}
					}
					str = str[0...-1]
					str << "};" << cr
					@countline += 1
				end
				
			}
			
			str << "// We can then define the surface as a list of line loops (only one" << cr
			str << "// here, since there are no holes--see `t4.geo'):" << cr
			@countsurface = 1
			0.upto(@loop.length - 1) do |p|
				str << "Plane Surface(#{@countsurface}) = {#{@loop[p]}};" << cr
				@countsurface +=1
			end

			# scrivo il file
			filename = UI.savepanel( "Save GMSH file ...", File.dirname(@model.path) , "#{@modelName}.geo" )
			fileGEO = File.new(filename, "w" )
			fileGEO.puts(str)
			fileGEO.close
			Sketchup.set_status_text("done")
		end
	end
end

if (not file_loaded?("su2gmsh.rb"))
	add_separator_to_menu("Tools")
	UI.menu("Tools").add_item("Export to GMSH *.geo") {SU2GMSH::Exporter.new}
end

file_loaded("su2gmsh.rb")
